/* http://www.codingunit.com/cplusplus-tutorial-polymorphism-and-abstract-base-class
One of the features of derived classes is that a pointer to a derived class is
type-compatible with a pointer to its base class. Polymorphism takes advantage of
this feature.

Lets take a look at an example of a base class and derived classes:

Also
http://www.terathon.com/code/vector2d.html
*/

//#include <iostream>
#include <cmath>

using namespace std;

// A 2D point class!
class Point2D {

private:
    // Coordinates of the point.
    double _x;
    double _y;

public:
    // Constructors
    Point2D() {};			// Default constructor
    Point2D(double x, double y) : _x(x), _y(y) {};

    // Destructor
    ~Point2D() {};

    // Accessor methods
    double getX() {return _x;}
    double getY() {return _y;}

    // Mutator methods
    void setX(double val) {_x = val;}
    void setY(double val) {_y = val;}

    // Distance operator
    double distance(Point2D& p) {return sqrt((_x-p._x)*(_x-p._x) + (_y-p._y)*(_y-p._y));}

    Point2D& operator =(const Point2D& v) { //overload assignment
        _x = v._x;
        _y = v._y;
        return (*this);
    }

    Point2D& operator *=(double t) {
        _x *= t;
        _y *= t;
        return (*this);
    }

    Point2D& operator /=(double t) {
        double d = 1.0D/t;
        _x *= d;
        _y *= d;
        return (*this);
    }

    Point2D operator -(void) const { //negation
        return (Point2D(-_x, -_y));
    }

    Point2D operator +(const Point2D& p) const
    {
        return (Point2D(_x + p._x, _y + p._y));
    }

    Point2D operator -(const Point2D& p) const
    {
        return (Point2D(_x - p._x, _y - p._y));
    }

    Point2D operator *(double t) const
    {
        return (Point2D(_x * t, _y * t));
    }

    Point2D operator /(double t) const
    {
        double d = 1.0D / t;
        return (Point2D(_x * d, _y * d));
    }
 };

class Triangle {
private:
    Point2D ABC[3];
public:
    Triangle(Point2D p1,Point2D p2,Point2D p3) {
       ABC[0] = p1;
       ABC[1] = p2;
       ABC[2] = p3;
    }

    double area() {
        double s[3];
        for(int i = 0; i < 3; i++)
            s[i] = ABC[i].distance(ABC[(i+1)%3]);
        double sp = (s[0]+s[1]+s[2])/2.;
        return sqrt(sp*(sp-s[0])*(sp-s[1])*(sp-s[2]));
    };

    //...
 };

class Polygon : public Triangle
{
	protected:
		Point2D* vertices;
		Triangle* dissection;


	public:
	    Polygon();
   	    Polygon(Triangle T);
        Polygon(Point2D* p);
		double area();
};

class Quad: public Triangle
{
public:
	Quad();
	Quad(Triangle);
	//..
	bool isRect();
	int area();

};




