// Sequence.h
#include <iostream>


class Sequence {
public:
    Sequence(long f = 0)
     : first(f), cur(f) {}
    virtual ~Sequence() {};
    void printSequence(int n);
protected:
    virtual long firstValue();
    virtual long nextValue() = 0;
    long first;
    long cur;
};

void Sequence::printSequence(int n) {
    std::cout << firstValue();
    for(int i = 2; i <= n; i++)
        std::cout << ' ' << nextValue();
}

long Sequence::firstValue() {
    cur = first;
    return cur;
}

long Sequence::nextValue() {
    return ++cur;
}

////=======================
//  Derived class ArithSequence
class ArithSequence : public Sequence {
public:
    ArithSequence(long i = 1);
    //ArithSequence(long i = 1, long j = 1)
protected:
    virtual long nextValue();
    long inc;
};

ArithSequence::ArithSequence(long i)
 : Sequence(), inc(i) { }

// could forgo the previous and just use this:
//ArithSequence::ArithSequence(long i)
// : Sequence(), inc(i) { }


long ArithSequence::nextValue() {
    cur += inc;
    return cur;
}

//==================
// geometric
class GeomSequence : public Sequence {
public:
    GeomSequence(long r = 2);
protected:
    virtual long nextValue();
    long ratio;
};

GeomSequence::GeomSequence(long r)
: Sequence(1), ratio(r) { }

long GeomSequence::nextValue() {
    cur *= ratio;
    return cur;
}

//===============
// Fibonacci
class FibonacciSequence : public Sequence { // Fibonacci Sequence
public:
    FibonacciSequence(long f = 0, long s = 1); // constructor
protected:
    virtual long firstValue(); // reset
    virtual long nextValue(); // advance
protected:
    long second; // second value
    long prev; // previous value
};


FibonacciSequence::FibonacciSequence(long f, long s)
  : Sequence(f), second(s), prev(second - first) { }
long FibonacciSequence::firstValue() { // reset
    cur = first;
    prev = second - first; // create fictitious prev
    return cur;
}
long FibonacciSequence::nextValue() { // advance
    long temp = prev;
    prev = cur;
    cur += temp;
    return cur;
}


